package com.miteksystems.misnapcontroller;

import android.util.Log;
import android.view.ViewGroup;

import androidx.annotation.Nullable;
import androidx.lifecycle.Observer;

import com.miteksystems.misnap.ControllerFragment;
import com.miteksystems.misnap.analyzer.AnalyzerFactory;
import com.miteksystems.misnap.analyzer.MiSnapAnalyzer;
import com.miteksystems.misnap.camera.MiSnapCamera;
import com.miteksystems.misnap.events.SetCaptureModeEvent;
import com.miteksystems.misnap.params.CameraApi;
import com.miteksystems.misnap.params.CameraParamMgr;
import com.miteksystems.misnap.params.DocType;
import com.miteksystems.misnap.params.ScienceParamMgr;

import org.json.JSONObject;

import static com.miteksystems.misnap.params.MiSnapApi.RESULT_ERROR_SDK_STATE_ERROR;

/**
 * Created by jlynch on 11/21/17.
 */

public class MiSnapFragment extends ControllerFragment {

    private static final String TAG = MiSnapFragment.class.getName();
    private MiSnapController miSnapController;
    private MiSnapAnalyzer analyzer;

    @Override
    protected void init() {
        super.init();
    }

    @Override
    protected void deinit() {
        super.deinit();
        if (miSnapController != null) {
            miSnapController.end();
            miSnapController = null;
        }

        if (analyzer != null) {
            analyzer.deinit();
            //no longer null because onOrientationChanged() could be called in a different thread than deinit()
//            analyzer = null;
        }
    }

    private MiSnapAnalyzer createAnalyzer(JSONObject parameters) {
        int whichAnalyzer;
        ScienceParamMgr paramMgr = new ScienceParamMgr(parameters);
        CameraParamMgr cameraParamMgr = new CameraParamMgr(parameters);
        if (paramMgr.isTestScienceCaptureMode()) {
            // Only used for building a test deck
            Log.e("Analyzer", "Creating TEST_SCIENCE_CAPTURE_ANALYZER");
            whichAnalyzer = AnalyzerFactory.TEST_SCIENCE_CAPTURE_ANALYZER;
        } else if (paramMgr.isTestScienceReplayMode()) {
            // Only used for analyzing a test deck
            Log.e("Analyzer", "Creating TEST_SCIENCE_REPLAY_ANALYZER");
            whichAnalyzer = AnalyzerFactory.TEST_SCIENCE_REPLAY_ANALYZER;
        } else if (new DocType(paramMgr.getRawDocumentType()).isCameraOnly()) {
            whichAnalyzer = AnalyzerFactory.NO_ANALYZER;
        } else {
            // Video capture mode
            Log.e("Analyzer", "Creating MISNAP_ANALYZER");
            whichAnalyzer = AnalyzerFactory.MISNAP_ANALYZER;
        }

        return AnalyzerFactory.createAnalyzer(whichAnalyzer, requireActivity(),
                OrientationUtils.getDocumentOrientation(requireActivity(), cameraParamMgr.getRequestedOrientation()),
                OrientationUtils.getDeviceOrientation(requireActivity()),
                parameters);
    }

    @Override
    public void initializeController() {
        try {
            //set the surfaceview in the main activity
            MiSnapCamera camera = cameraMgr.getMiSnapCamera();
            if (camera != null) {
                analyzer = createAnalyzer(miSnapParams);
                analyzer.init();
                miSnapController = new MiSnapController(requireActivity(), camera, analyzer, miSnapParams);

                // this is needed so that the frame can be returned in return intent
                miSnapController.getLiveDataResult().observe(this, new Observer<MiSnapControllerResult>() {
                    @Override
                    public void onChanged(@Nullable MiSnapControllerResult result) {
                        if (null == result) {
                            Log.w(TAG, "empty result");
                            return;
                        }

                        processFinalFrameMessage(result.getFinalFrame(), result.getFourCorners());

                        // it's possible for this fragment to go away while analyzer finishes processing a frame
                        if (null != cameraMgr) {
                            cameraMgr.receivedGoodFrame();
                        }
                    }
                });
                miSnapController.start();

                ((ViewGroup) getView()).addView(cameraMgr.getSurfaceView());

            } else {
                handleErrorState(RESULT_ERROR_SDK_STATE_ERROR);
            }
        } catch (Exception e) {
            Log.e(TAG, e.toString());
            handleErrorState(RESULT_ERROR_SDK_STATE_ERROR);
        }
    }

    @Override
    public void onEvent(SetCaptureModeEvent event) {
        if (CameraApi.PARAMETER_CAPTURE_MODE_MANUAL == event.mode
                && camParamsMgr.isCurrentModeVideo()) {
            analyzer.deinit();
        } else if (CameraApi.PARAMETER_CAPTURE_MODE_AUTO == event.mode
                && !camParamsMgr.isCurrentModeVideo()) {
            analyzer.init();
        }

        super.onEvent(event);
    }
}
