package com.miteksystems.misnapextractioncontroller;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;

import com.miteksystems.misnap.natives.MiSnapScience;
import com.miteksystems.misnap.params.MiSnapApi;
import com.miteksystems.misnap.utils.Utils;

public class OrientationUtils {
    private static final int MISNAP_ORIENTATION_COUNT = 4;

    public static int cameraRotationToNativeOrientation(int orientation) {
        int result;
        switch (orientation) {
            case 90:
                result = MiSnapScience.Orientation.PORTRAIT;
                break;
            case 270:
                result = MiSnapScience.Orientation.REVERSE_PORTRAIT;
                break;
            case 180:
                result = MiSnapScience.Orientation.LANDSCAPE_RIGHT;
                break;
            case 0:
            default:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
        }
        return result;
    }

    public static int deviceOrientationToNativeOrientation(int orientation) {
        int result;
        switch (orientation) {
            case ActivityInfo.SCREEN_ORIENTATION_PORTRAIT:
                result = MiSnapScience.Orientation.PORTRAIT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE:
                result = MiSnapScience.Orientation.LANDSCAPE_RIGHT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT:
                result = MiSnapScience.Orientation.REVERSE_PORTRAIT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE:
            default:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
        }
        return result;
    }

    public static int getDocumentOrientation(Context activityContext, int requestedOrientation) {
        int orientation = cameraRotationToNativeOrientation(Utils.getCameraRotationDegrees(activityContext));
        if (shouldRotateOrientation90(activityContext, requestedOrientation)) {
            orientation = (orientation + 1) % MISNAP_ORIENTATION_COUNT;  //rotate 90 degrees
        }
        return orientation;
    }

    public static int getDeviceOrientation(Context activityContext) {
        return cameraRotationToNativeOrientation(Utils.getCameraRotationDegrees(activityContext));
    }

    private static boolean shouldRotateOrientation90(Context activityContext, int requestedOrientation) {
        return ((requestedOrientation == MiSnapApi.PARAMETER_ORIENTATION_DEVICE_PORTRAIT_DOCUMENT_PORTRAIT
                || requestedOrientation == MiSnapApi.PARAMETER_ORIENTATION_DEVICE_FREE_DOCUMENT_ALIGNED_WITH_DEVICE)
                && Utils.getDeviceBasicOrientation(activityContext) == Configuration.ORIENTATION_PORTRAIT);
    }
}
