package com.miteksystems.misnaphybridcontroller;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.util.Log;
import android.view.ViewGroup;

import androidx.annotation.Nullable;
import androidx.lifecycle.Observer;

import com.miteksystems.misnap.ControllerFragment;
import com.miteksystems.misnap.analyzer.MiSnapAnalyzer;
import com.miteksystems.misnap.barcode.analyzer.BarcodeAnalyzer;
import com.miteksystems.misnap.barcode.analyzer.MiSnapBarcodeDetector;
import com.miteksystems.misnap.camera.MiSnapCamera;
import com.miteksystems.misnap.events.OnCapturedFrameEvent;
import com.miteksystems.misnap.events.SetCaptureModeEvent;
import com.miteksystems.misnap.natives.MiSnapScience;
import com.miteksystems.misnap.params.BarcodeApi;
import com.miteksystems.misnap.params.CameraApi;
import com.miteksystems.misnap.params.CameraParamMgr;
import com.miteksystems.misnap.params.MiSnapApi;
import com.miteksystems.misnap.utils.Utils;

import org.greenrobot.eventbus.EventBus;
import org.json.JSONObject;

import static com.miteksystems.misnap.params.MiSnapApi.RESULT_ERROR_SDK_STATE_ERROR;

public class MiSnapHybridFragment extends ControllerFragment {

    private static final String TAG = MiSnapHybridFragment.class.getName();
    private MiSnapHybridController controller;
    private MiSnapAnalyzer miSnapAnalyzer;
    private BarcodeAnalyzer barcodeAnalyzer;
    private MiSnapBarcodeDetector miSnapBarcodeDetector;

    @Override
    protected void init() {
        super.init();
    }

    @Override
    protected void deinit() {
        super.deinit();
        if (controller != null) {
            controller.end();
            controller = null;
        }

        if (miSnapAnalyzer != null) {
            miSnapAnalyzer.deinit();
            //no longer null because onOrientationChanged() could be called in a different thread than deinit()
//            analyzer = null;
        }

        if (barcodeAnalyzer != null) {
            barcodeAnalyzer.deinit();
            //no longer null because onOrientationChanged() could be called in a different thread than deinit()
//            analyzer = null;
        }

        if (miSnapBarcodeDetector != null) {
            miSnapBarcodeDetector.deinit();
        }
    }

    @Override
    public void initializeController() {
        try {
            //set the surfaceview in the main activity
            MiSnapCamera camera = cameraMgr.getMiSnapCamera();
            if (camera != null) {
                Context appContext = requireActivity().getApplicationContext();
                Context activityContext = requireActivity();
                int orientation = new CameraParamMgr(miSnapParams).getRequestedOrientation();
                miSnapAnalyzer = new MiSnapAnalyzer(appContext, miSnapParams, OrientationUtils.getDlDocumentOrientation(activityContext, orientation), OrientationUtils.getDlDeviceOrientation(activityContext), false);
                JSONObject customParams = new JSONObject(miSnapParams.toString());
                customParams.put(MiSnapApi.MiSnapDocumentType, MiSnapApi.PARAMETER_DOCTYPE_PDF417);
                barcodeAnalyzer = new BarcodeAnalyzer(appContext, customParams, Utils.getDeviceBasicOrientation(activityContext), OrientationUtils.getBarcodeDocumentOrientation(activityContext, orientation));
                miSnapBarcodeDetector = new MiSnapBarcodeDetector();
                miSnapAnalyzer.init();
                barcodeAnalyzer.init();
                miSnapBarcodeDetector.init();
//                barcodeAnalyzer.setScanningSpeed(BarcodeApi.BARCODE_SPEED_SLOW);
                controller = new MiSnapHybridController(activityContext, camera, miSnapAnalyzer, barcodeAnalyzer, miSnapBarcodeDetector, miSnapParams);

                // this is needed so that the frame can be returned in return intent
                controller.getResult().observe(this, new Observer<MiSnapHybridControllerResult>() {
                    @Override
                    public void onChanged(@Nullable MiSnapHybridControllerResult result) {
                        if (null == result) {
                            Log.w(TAG, "empty result");
                            return;
                        }

                        handleFinalFrame(result);

                        // it's possible for this fragment to go away while analyzer finishes processing a frame
                        if (null != cameraMgr) {
                            cameraMgr.receivedGoodFrame();
                        }
                    }
                });
                controller.start();

                ((ViewGroup) getView()).addView(cameraMgr.getSurfaceView());

            } else {
                handleErrorState(RESULT_ERROR_SDK_STATE_ERROR);
            }
        } catch (Exception e) {
            Log.e(TAG, e.toString());
            handleErrorState(RESULT_ERROR_SDK_STATE_ERROR);
        }
    }

    @Override
    public void onEvent(SetCaptureModeEvent event) {
        if (CameraApi.PARAMETER_CAPTURE_MODE_MANUAL == event.mode
                && camParamsMgr.isCurrentModeVideo()) {
            miSnapAnalyzer.deinit();
//            barcodeAnalyzer.deinit();
        } else if (CameraApi.PARAMETER_CAPTURE_MODE_AUTO == event.mode
                && !camParamsMgr.isCurrentModeVideo()) {
            miSnapAnalyzer.init();
//            barcodeAnalyzer.init();
        }

        super.onEvent(event);
    }

    protected int cameraRotationToNativeOrientation(int orientation) {
        int result;
        switch (orientation) {
            case 90:
                result = MiSnapScience.Orientation.PORTRAIT;
                break;
            case 0:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
            case 270:
                result = MiSnapScience.Orientation.REVERSE_PORTRAIT;
                break;
            case 180:
                result = MiSnapScience.Orientation.LANDSCAPE_RIGHT;
                break;
            default:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
        }
        return result;
    }

    protected int deviceOrientationToNativeOrientation(int orientation) {
        int result;
        switch (orientation) {
            case ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_PORTRAIT:
                result = MiSnapScience.Orientation.PORTRAIT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE:
                result = MiSnapScience.Orientation.LANDSCAPE_RIGHT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT:
                result = MiSnapScience.Orientation.REVERSE_PORTRAIT;
                break;
            default:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
        }
        return result;
    }

    private void handleFinalFrame(MiSnapHybridControllerResult result) {
        String resultCode;
        if (camParamsMgr.isCurrentModeVideo()) {
            resultCode = MiSnapApi.RESULT_SUCCESS_VIDEO;
            mWarnings.clear();
        } else {
            resultCode = MiSnapApi.RESULT_SUCCESS_STILL;
        }


        Intent returnIntent = buildReturnIntent(Activity.RESULT_OK, result.getFinalFrame(), resultCode, result.getFourCorners());
        if (!result.getExtractedBarcode().isEmpty()) {
            returnIntent.putExtra(BarcodeApi.RESULT_PDF417_DATA, result.getExtractedBarcode());
            returnIntent.putExtra(BarcodeApi.RESULT_RAW_DATA, result.getRawBarcode());
        }
        EventBus.getDefault().post(new OnCapturedFrameEvent(returnIntent));
    }
}
