package com.miteksystems.misnaphybridcontroller;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;

import com.miteksystems.misnap.natives.MiSnapScience;
import com.miteksystems.misnap.params.MiSnapApi;
import com.miteksystems.misnap.utils.Utils;

public class OrientationUtils {
    private static final int MISNAP_ORIENTATION_COUNT = 4;

    public static int cameraRotationToNativeOrientation(int orientation) {
        int result;
        switch (orientation){
            case 90:
                result =  MiSnapScience.Orientation.PORTRAIT;
                break;
            case 0:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
            case 270:
                result = MiSnapScience.Orientation.REVERSE_PORTRAIT;
                break;
            case 180:
                result = MiSnapScience.Orientation.LANDSCAPE_RIGHT;
                break;
            default:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
        }
        return result;
    }

    public static int deviceOrientationToNativeOrientation(int orientation) {
        int result;
        switch (orientation) {
            case ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_PORTRAIT:
                result = MiSnapScience.Orientation.PORTRAIT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE:
                result = MiSnapScience.Orientation.LANDSCAPE_RIGHT;
                break;
            case ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT:
                result = MiSnapScience.Orientation.REVERSE_PORTRAIT;
                break;
            default:
                result = MiSnapScience.Orientation.LANDSCAPE_LEFT;
                break;
        }
        return result;
    }

    public static int getDlDocumentOrientation(Context activityContext, int requestedOrientation) {

        int orientation = cameraRotationToNativeOrientation(Utils.getCameraRotationDegrees(activityContext));
        if (shouldRotateOrientation90(activityContext, requestedOrientation)) {
            orientation = (orientation + 1) % MISNAP_ORIENTATION_COUNT;  //rotate 90 degrees
        }
        return orientation;
    }

    public static int getDlDeviceOrientation(Context activityContext) {
        return cameraRotationToNativeOrientation(Utils.getCameraRotationDegrees(activityContext));
    }

    public static boolean shouldRotateOrientation90(Context activityContext, int requestedOrientation) {
        return ((requestedOrientation == MiSnapApi.PARAMETER_ORIENTATION_DEVICE_PORTRAIT_DOCUMENT_PORTRAIT
                || requestedOrientation == MiSnapApi.PARAMETER_ORIENTATION_DEVICE_FREE_DOCUMENT_ALIGNED_WITH_DEVICE)
                && Utils.getDeviceBasicOrientation(activityContext) == Configuration.ORIENTATION_PORTRAIT);
    }

    public static int getBarcodeDocumentOrientation(Context activityContext, int requestedOrientation) {
        int result = Configuration.ORIENTATION_LANDSCAPE;
        if (Utils.getDeviceBasicOrientation(activityContext) == Configuration.ORIENTATION_PORTRAIT) {
            if (requestedOrientation == MiSnapApi.PARAMETER_ORIENTATION_DEVICE_FREE_DOCUMENT_ALIGNED_WITH_DEVICE ||
                    requestedOrientation == MiSnapApi.PARAMETER_ORIENTATION_DEVICE_PORTRAIT_DOCUMENT_PORTRAIT) {
                result = Configuration.ORIENTATION_PORTRAIT;
            }
        }
        return result;
    }

    public static int getBarcodeDeviceOrientation(Context activityContext) {
        return Utils.getDeviceBasicOrientation(activityContext);
    }
}
